//-------------------------------------------------------------------------
// Titel : einfaches Beispielprogramm f�r den mySmartControl M168
//-------------------------------------------------------------------------
// Funktion : einfaches Terminalprogram, schaltet Ports und I/O-Lines
// Schaltung : USB an PC, Virtueller Comport, COMx, 9600 Baud, 8,n,1
//-------------------------------------------------------------------------
// Prozessor : ATmega168
// Takt : 20000000 Hz
// Sprache : C
// Datum : 16.11.2007
// Version : 1.2
//-------------------------------------------------------------------------
// created by myAVR-CodeWizard
//-------------------------------------------------------------------------
#define 	F_CPU 	20000000L
#include 	<avr/io.h>
#include	<stdbool.h>
#define 	BAUD 	115200L
#define		UBRR_VAL 10
// #define		UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)
#define		BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))
#define		BAUD_ERROR 0
//#define		BAUD_ERROR ((BAUD_REAL*1000)/BAUD-1000)

#if ((BAUD_ERROR>10) || (BAUD_ERROR<-10))
  #error Systematischer Fehler der Baudrate grösser 1% und damit zu hoch!
#endif


//----------------------------------------------------------------------
// Titel 	: C-Funktion Zeichen zu UART senden.
// IN  	 	: char data
//----------------------------------------------------------------------
void uartPutChar(char data)
{
	//warte bis UDR leer ist UCSRA / USR bei z.B.: 2313
	while (!(UCSR0A&32));
	//sende
	UDR0=data;
}
void uartPutBin(char data)
{
	//warte bis UDR leer ist UCSRA / USR bei z.B.: 2313
	while (!(UCSR0A&32));
	//sende
if (data&1)   uartPutChar('1'); else uartPutChar('0');
if (data&2)   uartPutChar('1'); else uartPutChar('0');
if (data&4)   uartPutChar('1'); else uartPutChar('0');
if (data&8)   uartPutChar('1'); else uartPutChar('0');
if (data&16)  uartPutChar('1'); else uartPutChar('0');
if (data&32)  uartPutChar('1'); else uartPutChar('0');
if (data&64)  uartPutChar('1'); else uartPutChar('0');
if (data&128) uartPutChar('1'); else uartPutChar('0');
}
//----------------------------------------------------------------------
// Titel 	: C-Funktion wartet auf Zeichen von UART.
// OUT 	 	: data
//----------------------------------------------------------------------
char uartGetChar()
{
	char data=0;
	//warte bis RX-complete RXC UCSRA / USR bei z.B.: AT090S2313
	while (!(UCSR0A&128));
	//empfangen
	data=UDR0;
	return data;
}
//----------------------------------------------------------------------
// Titel 	: C-Funktion wartet auf Zeichen von UART.
// OUT 	 	: data, error = keine Daten
//----------------------------------------------------------------------
bool error;
char uartIsChar()
{
	char data=0;
	error=true;
	//chek RX-complete RXC UCSRA
	if (!(UCSR0A&128)) return 0;
	//zeihen abholen
	data=UDR0;
	error=false;
	return data;
}
//----------------------------------------------------------------------
// Titel 	: C-Funktion Zeichenkette zu UART senden.
// IN  	 	: char *buffer, Zeichenkette mit NUll abgeschlossen
//----------------------------------------------------------------------
void uartPutString(char *buffer)
{
	int i;
	for (i=0; buffer[i] !=0;i++)
		uartPutChar (buffer[i]);
}
//------------------------------------------------------------------------
// Initialisierungen
//------------------------------------------------------------------------
void init()
{
	// UART initialisieren
//	sbi(UCSR0B,3); 	// TX aktiv
//	sbi(UCSR0B,4); 	// RX aktivieren
	UCSR0B |= ( 1 << 3 );  // TX aktiv
	UCSR0B |= ( 1 << 4 );  // RX aktivieren
	UBRR0H = UBRR_VAL >> 8;
	UBRR0L = UBRR_VAL & 0xFF;
//	UBRR0L=(uint8_t)(F_CPU/(BAUD*16L))-1; 	// Baudrate festlegen
//	UBRR0H=(uint8_t)((F_CPU/(BAUD*16L))-1)>>8; 	// Baudrate festlegen
	//alles Ausgang
	DDRB=0x3F;
	DDRC=0xFF;
	DDRD=0xFE;
}
// Willkommen- und Hilfebeildschirm
void intro()
{
	int i;
	uartPutChar('\r');
	uartPutChar('\r');
	for (i=0;i<42;i++) uartPutChar('=');
	uartPutString("\r mySmartControl Simple Term 1.2.1 \r www.myAVR.de \r modified by Andreas Klafft");
	uartPutChar('\r');
	for (i=0;i<42;i++) uartPutChar('-');
	uartPutString("\rKommandos: zwei Byte, Kein Trennzeichen");
	uartPutString("\r PB = PortB alle Bits ON");
	uartPutString("\r pb = PortB alle Bits OFF");
	uartPutString("\r PC = PortC alle Bits ON");
	uartPutString("\r pc = PortC alle Bits OFF");
	uartPutString("\r PD = PortD alle Bits ON");
	uartPutString("\r Bx = PortB Bit X ON  (X = 0..5)");
	uartPutString("\r bx = PortB Bit X OFF (X = 0..5)");
	uartPutString("\r Cx = PortC Bit X ON  (X = 0..5)");
	uartPutString("\r cx = PortC Bit X OFF (X = 0..5)");
	uartPutString("\r Dx = PortD Bit X ON  (X = 2..7)");
	uartPutString("\r dx = PortD Bit X OFF (X = 2..7)");
	uartPutString("\r IX = config Port(C,B,D) IN mit PullUP");
	uartPutString("\r ix = config Port(c,b,d) IN ohne PullUP");
	uartPutString("\r gx = get PIN(b,c,d)");
	uartPutString("\r GX = get Port(B,C,D)");
	uartPutString("\r Gx = get DDR(b,c,d)");
	uartPutString("\r Ox = config Port(B,C,D) = OUT");
	uartPutChar('\r');
	for (i=0;i<42;i++) uartPutChar('=');
	uartPutString("\rAtmega168> ");
}
/////////////////////////////////////////////////////////////////////////////
// Main-Funktion
/////////////////////////////////////////////////////////////////////////////
main()
{
	unsigned char portBuffer[3]={0}; // B,C,D
	char z1,z2;
	init(); 	// Initialisierungen
	intro();
	while (true)  	// Mainloop-Begin
	{
		// Kommandos abfragen
		z1=uartIsChar();
		if (error) continue;
		uartPutChar(z1);
		if (z1=='?')
		{
			intro();
			continue;
		}
		z2=uartGetChar();
		if (error) continue;
		uartPutChar(z2);
		uartPutChar('\r');
		if (z1=='?')
		{
			intro();
			continue;
		}
		switch (z1)
		{
			case 'I':
			switch(z2)
			{
				case 'B':
				DDRB=0x00;
				PORTB=0xFF;
				break;	
				case 'C':
				DDRC=0x00;
				PORTC=0xFF;
				break;	
				case 'D':
				DDRD=0x02;
				PORTD=0xFC;
				break;	
				default:
				uartPutChar('?');
				break;	
			}

			break;	
			case 'i':
			switch(z2)
			{
				case 'B':
				DDRB=0x00;
				PORTB=0x00;
				break;	
				case 'C':
				DDRC=0x00;
				PORTC=0x00;
				break;	
				case 'D':
				DDRD=0x02;
				PORTD=0x00;
				break;	
				default:
				uartPutChar('?');
				break;	
			}

			break;	

			case 'g':
			switch(z2)
			{
				case 'B':
				case 'b':
				uartPutString("PinB=");
				uartPutBin(PINB);
				break;	
				case 'C':
				case 'c':
				uartPutString("PinC=");
				uartPutBin(PINC);
				break;	
				case 'D':
				case 'd':
				uartPutString("PinD=");
				uartPutBin(PIND);
				break;	
				default:
				uartPutChar('?');
				break;	
			}
			break;	
			case 'G':
			switch(z2)
			{
				case 'B':
				uartPutString("\rPortB=");
				uartPutBin(PINB);
				break;	
				case 'C':
				uartPutString("\rPortC=");
				uartPutBin(PINC);
				break;	
				case 'D':
				uartPutString("\rPortD=");
				uartPutBin(PIND);
				break;	
				case 'b':
				uartPutString("\rDDRB=");
				uartPutBin(DDRB);
				break;	
				case 'c':
				uartPutString("\rDDRC=");
				uartPutBin(DDRC);
				break;	
				case 'd':
				uartPutString("\rDDRD=");
				uartPutBin(DDRD);
				break;	
				default:
				uartPutChar('?');
				break;	
			}
			break;	

			case 'O':
			case 'o':
			switch(z2)
			{
				case 'B':
				DDRB=0x3F;
				break;	
				case 'C':
				DDRC=0xFF;
				break;	
				case 'D':
				DDRD=0xFE;
				break;	
				default:
				uartPutChar('?');
				break;	
			}

			break;	

			case 'P':
			case 'p':
			switch(z2)
			{
				case 'B':
				PORTB=0xFF;
				break;
				case 'C':
				PORTC=0xFF;
				break;
				case 'D':
				PORTD=0xFF;
				break;
				case 'b':
				PORTB=0x00;
				break;
				case 'c':
				PORTC=0x00;
				break;
				case 'd':
				PORTD=0x00;
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			case 'B':
			switch(z2)
			{
				case '0':
//				sbi(PORTB,0);
				PORTB |= (1<<0);
				break;
				case '1':
//				sbi(PORTB,1);
				PORTB |= (1<<1);
				break;
				case '2':
//				sbi(PORTB,2);
				PORTB |= (1<<2);
				break;
				case '3':
//				sbi(PORTB,3);
				PORTB |= (1<<3);
				break;
				case '4':
//				sbi(PORTB,4);
				PORTB |= (1<<4);
				break;
				case '5':
//				sbi(PORTB,5);
				PORTB |= (1<<5);
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			case 'C':
			switch(z2)
			{
				case '0':
//				sbi(PORTC,0);
				PORTC |= (1<<0);
				break;
				case '1':
//				sbi(PORTC,1);
				PORTC |= (1<<1);
				break;
				case '2':
//				sbi(PORTC,2);
				PORTC |= (1<<2);
				break;
				case '3':
//				sbi(PORTC,3);
				PORTC |= (1<<3);
				break;
				case '4':
//				sbi(PORTC,4);
				PORTC |= (1<<4);
				break;
				case '5':
//				sbi(PORTC,5);
				PORTC |= (1<<5);
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			case 'D':
			switch(z2)
			{
				case '2':
//				sbi(PORTD,2);
				PORTD |= (1<<2);
				break;
				case '3':
//				sbi(PORTD,3);
				PORTD |= (1<<3);
				break;
				case '4':
//				sbi(PORTD,4);
				PORTD |= (1<<4);
				break;
				case '5':
//				sbi(PORTD,5);
				PORTD |= (1<<5);
				break;
				case '6':
//				sbi(PORTD,6);
				PORTD |= (1<<6);
				break;
				case '7':
//				sbi(PORTD,7);
				PORTD |= (1<<7);
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			case 'b':
			switch(z2)
			{
				case '0':
//				cbi(PORTB,0);
				PORTB &= ~(1<<0);
				break;
				case '1':
//				cbi(PORTB,1);
				PORTB &= ~(1<<1);
				break;
				case '2':
//				cbi(PORTB,2);
				PORTB &= ~(1<<2);
				break;
				case '3':
//				cbi(PORTB,3);
				PORTB &= ~(1<<3);
				break;
				case '4':
//				cbi(PORTB,4);
				PORTB &= ~(1<<4);
				break;
				case '5':
//				cbi(PORTB,5);
				PORTB &= ~(1<<5);
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			case 'c':
			switch(z2)
			{
				case '0':
//				cbi(PORTC,0);
				PORTC &= ~(1<<0);
				break;
				case '1':
//				cbi(PORTC,1);
				PORTC &= ~(1<<1);
				break;
				case '2':
//				cbi(PORTC,2);
				PORTC &= ~(1<<2);
				break;
				case '3':
//				cbi(PORTC,3);
				PORTC &= ~(1<<3);
				break;
				case '4':
//				cbi(PORTC,4);
				PORTC &= ~(1<<4);
				break;
				case '5':
//				cbi(PORTC,5);
				PORTC &= ~(1<<5);
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			case 'd':
			switch(z2)
			{
				case '2':
//				cbi(PORTD,2);
				PORTD &= ~(1<<2);
				break;
				case '3':
//				cbi(PORTD,3);
				PORTD &= ~(1<<3);
				break;
				case '4':
//				cbi(PORTD,4);
				PORTD &= ~(1<<4);
				break;
				case '5':
//				cbi(PORTD,5);
				PORTD &= ~(1<<5);
				break;
				case '6':
//				cbi(PORTD,6);
				PORTD &= ~(1<<6);
				break;
				case '7':
//				cbi(PORTD,7);
				PORTD &= ~(1<<7);
				break;
				default:
				uartPutChar('?');
				break;	
			}
			break;
			default:
			uartPutChar('?');
			break;
		}
		uartPutString("\rAtmega168> ");
	}
	// Mainloop-Ende
}
//---------------------------------------------------------------------------
